### USER-DEFINED SECTION; edit these values ###
DataFile = "pvalues-all.csv.gz"
# lower and upper bounds for the number of true effects
lower.m = 1
upper.m = 3000
# uncomment one of 3 following lines
d <- read.csv(gzfile(DataFile), as.is=TRUE) # if DataFile is gzip-ed csv  (*.csv.gz)
#d <- read.csv(unzip(DataFile), as.is=TRUE) # if DataFile is zip-ed csv (*.zip.csv)
#d <- read.csv(DataFile, as.is=TRUE) # if DataFile is non-compressed csv file
#
# alpha-threshold for p-values used for estimation
# negative value indicates that alpha will be computed in data-dependent manner
alpha = -1 # data-dependent alpha
# uncomment next line if want alpha in the form i/(k+1), expectation of ith p under H0
# alpha = 50 / (dim(d)[1] + 1)
source("fit3par-interval.r")
# If the distribution of the bulk of p-values is not sufficiently uniform
# it is possible to adjust the null distribution in an interval
# the assumption below is that majority of p-values in the "adj.interval" interval
# are not true associations and deviate from uniformity due to a systematic component
# (e.g. population stratificatin).
# Uncomment (and possibly edit) 9 lines below if want to do the null distribution adjustment
 d = d[order(d$Pvalue),]
 adj.interval = c(1e-5, 0.05) # p-value interval from which to infer the systematic component
 df = 1 # assume that p-values come from tests with "df" degrees of freedom
 nc = c(0, 1, 20) # boundaries and midvalue for noncentrality inflation due to the systematic component
 sc = c(1, 1.1, 2) # boundaries and midvalue for scale inflation due to the systematic component
 g <- Fit3par.interval(p=d$Pvalue, nc=nc, df=c(1), sc=sc, trunc=adj.interval)
 adj.p = array(0, k <- length(d$Pvalue))
 for(i in 1 : length(d$Pvalue)) adj.p[i] = pdist(d$Pvalue[i], g$nc, g$df, g$sc)
 d$Pvalue = adj.p
## END OF USER-DEFINED SECTION

p <- d$Pvalue
# min p-value converted to noncentrality gives a biased upward effect size,
# appropriate for setting the gamma truncation parameter
tr <- 10 + qchisq(min(p), df=1, lower.tail=FALSE)
k <- length(p)

par.low <- c(0.634, 2.0)
par.upp <- c(0.99, 100)
par.mid <- c(0.95, 20)

pr.low <- 1 - upper.m/k 
pr.upp <- 1 - lower.m/k
pr.mid <- mean(c(pr.low, pr.upp))

max.iter <- 250
cov.check <- 0.002

Get.Alpha <- function(pv) {
  ord.max <- 100
  ratio <- 0
  num.ta <- 15
  k <- length(pv)
  j <- 1
  repeat{
    epi <- j/(k+1)
    x <- length(pv[pv<epi]) - k*epi
    ta.fa <- x/(epi*k)
    if(x >= num.ta && ta.fa >= ratio) break;
    j <- j+1 
    if(j>ord.max) {
      cat("warning: p-values are too close to uniform\n")
      break;
    } 
  }
  alpha <- epi
  return (alpha)
}

# pdf of p-values given a non-centrality
fy.nc <- function(nc, y, df=1) {
  ar <- qchisq(y, df=df, lower.tail=FALSE)
  dchisq(ar, df=df, ncp=nc) / dchisq(ar, df=df)
}

# pdf of a truncated gamma
h <- function(nc, a, b, tr) {
  if(tr < Inf) {
    rv <- dgamma(nc, shape=a, scale=b) / pgamma(tr, shape=a, scale=b)
  } else {
    rv <- dgamma(nc, shape=a, scale=b)
  }
  return (rv)
}

# pdf of top p-values for true effects which may have different non-centralities
# for h(nc)*f(p|nc)
# pull out the denominator of f(p|nc) 
# drop it for now
fy.int <- function(nc, a, b, tr, chi) {
  h(nc,a,b,tr) * dchisq(chi, df=1, ncp=nc)
}

# h(nc)*F(p|nc)
Fy.int <- function(nc, a, b, tr, chi) {
  h(nc,a,b,tr) * (1-pchisq(chi, df=1, ncp=nc))
}

# numerator of Pr(p|p<alpha)
fy <- function(pv, a, b, tr) {
  chi <- qchisq(pv, df=1, lower.tail=FALSE)
  marg <- integrate(fy.int,0,tr,a,b,tr,chi, rel.tol=1e-8)$value / dchisq(chi, df=1) # put back denominator of f(p|nc)
  return (marg)
}

# denominator of Pr(p|p<alpha)
Fy <- function(pv, a, b, tr) {
  chi <- qchisq(pv, df=1, lower.tail=FALSE)
  marg <- integrate(Fy.int,0,tr,a,b,tr,chi, rel.tol=1e-8)$value
  return (marg)
}

# vectorized numerators of Pr(p|p<alpha)
vec.fy <- function(pv, a, b, tr) {
  sapply(pv, fy, a, b, tr)
}

# vectorized denominators of Pr(p|<alpha)
vec.Fy <- function(pv, a, b, tr) {
  sapply(pv, Fy, a, b, tr)
}

# random numbers from right-truncated gamma density
frnd.dist <- function(n, Prior) {
    x <- runif(n)
    xx <- x*pgamma(Prior[3], shape=Prior[1], scale=Prior[2]) # Prior[3]: truncated value
    qgamma(xx, shape=Prior[1], scale=Prior[2])
}

# random p-values for true effects
Rpns <- function(n, Prior) {
  repeat {
    ncent <- frnd.dist(n, Prior) # sampling random noncentralities
    z <- rnorm(n, mean=sqrt(ncent), sd=1)
    pv <- pchisq(z^2, df=1, lower.tail = FALSE)
    if(min(pv) > 0) return (pv)
  }
}

# ------------
# EM-algorithm
# ------------

# function to optimize with respect to a and b in each iteration of the EM
optim.2par <- function(pars, last, pvals, tr, alpha) 
{
  a <- pars[1]; b <- pars[2];

  # get t1 and t2	
  p.last <- last[1]
  a.last <- last[2]
  b.last <- last[3]

  f1 <- 1/alpha
  f2 <- vec.fy(pvals, a.last, b.last, tr)/vec.Fy(alpha, a.last, b.last, tr) 

  t1 <- (p.last*f1)/(p.last*f1+(1-p.last)*f2)
  t2 <- 1-t1
  -sum(t2*(log(vec.fy(pvals,a,b,tr))-log(Fy(alpha,a,b,tr))))
}

# log likelihood
maximized <- function(p, a, b, pvals, alpha, tr)
{
  f1 <- 1/alpha
  f2 <- vec.fy(pvals, a, b, tr)/vec.Fy(alpha, a, b, tr) 
  sum(log(p*f1+(1-p)*f2))
}


# EM main function
EM <- function(par0, tr, alpha, pvals)
{
  # initial parameter values
  p <- par0[1]
  a <- par0[2]
  b <- par0[3]

  # initial log likelihood of y
  max <- maximized(p, a, b, pvals, alpha, tr)

  # current log likelihood of y minus last log likelihood of y
  # initial value set to be any value > cov.check (here, 10)
  # when smaller than cov.check, converge!	
  diff <- 10
        
  # iter: number of iterations
  # number of convergences among the iterations
  iter <- ccount <- 0

  while(diff>cov.check) {
    iter <- iter+1

    # pdf of p-values for false effects
    f1 <- 1/alpha

    # pdf of p-values for true effects
    f2 <- vec.fy(pvals, a, b, tr)/vec.Fy(alpha, a, b, tr) 

    # estimated false effect probabilities
    t1 <- (p*f1)/(p*f1+(1-p)*f2)
    t2 <- 1-t1

    # update p
    p.new <- mean(t1,rm.na=TRUE)

    # update a and b
    op <- try(optim(fn=optim.2par, method="L-BFGS-B",
                    par = par.mid, lower=par.low, upper=par.upp,
                    last=c(p,a,b), pvals=pvals, tr=tr, alpha=alpha))

    if(op$convergence == 0) {
      ccount <- ccount+1
      a.new <- op$par[1] 
      b.new <- op$par[2]
      max.last <- max
      max <- maximized(p.new, a.new, b.new, pvals, alpha, tr)
      diff <- max-max.last
      p <- p.new
      a <- a.new
      b <- b.new
    } else { # if op doesn't converge, stop iterations and return NAs ...
      p <- a <- b <- NA 
      break;
    }
    if(iter==max.iter) { # if the EM iterates up to max.iter, stop iterations and return NAs ...
      p <- a <- b <- NA			
      break;
    }  
  }

  return(c(p, a, b, iter, ccount))
}

options(error=quote({dump.frames(to.file=FALSE)}))
if(alpha <= 0) alpha = Get.Alpha(p) 
ptr <- p[p <= alpha]
em.out <- EM(par0=c(pr.mid, par.mid), tr=tr, alpha=alpha, pvals=ptr)
qq.em <- em.out[1]
aa.em <- em.out[2]
bb.em <- em.out[3]
ta.em <- if(!is.na(aa.em) & !is.na(bb.em)) {
  length(ptr)*(1-qq.em)/Fy(alpha,aa.em,bb.em,tr)
} else { NA }

iter.em <- em.out[4]
nc.em <- em.out[4]-em.out[5]

cat("\n")
cat("alpha =", alpha, "\n")
cat("number of p-values <= alpha:", length(ptr), "\n")
cat("non-convergence rate (em-2par):", nc.em/iter.em,"\n")
cat("number of iterations (em):", iter.em,"\n")
cat("estimated proportion of true effects:", ta.em/k, "\n")
cat("simulating ncs from the gamma with mean shape and mean scale...\n")
ncs <- frnd.dist(1e5, c(aa.em, bb.em, tr))
cat("em: est mean of ncs=", mean(ncs), "est sd of ncs=", sd(ncs), "\n\n")
cat("Effect distribution parameter estimates:", "Shape=", aa.em, "Scale=", bb.em, "\n")
cat("estimated number of true effects:", ta.em, "\n")

qShape = aa.em
qScale = bb.em
At = tr
Nloc = rep(1, round(ta.em))
